/*******************************************************************************
 * Copyright 2012 pw999
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.oqt.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import net.sf.oqt.model.ResultVO;

import org.apache.maven.project.MavenProject;

/**
 * The class centralizes the serialization process of the ResultVO.
 * <p>
 * In order to transfer the information from the plugin to the reporting plugin, the data is serialized to disk. This
 * also allows developers to deserialize the object and use the information in other plugins.
 * 
 * @author phillip
 * 
 */
public final class Serializer {
    /**
     * Where the file is saved.
     */
    private final File resultFile;
    
    private final File outputDir;

    /**
     * Creates a new Serializer object. This will initialize the file where the ResultVO will be stored.
     * <p>
     * The directory where the object will be saved will also be created.
     * 
     * @param project a reference to the Maven project.
     * @throws IOException if failed to create the output directory
     */
    public Serializer(final MavenProject project) throws IOException {
        File buildDir = new File(project.getBuild().getDirectory());
        if (!buildDir.exists() && !buildDir.mkdir()) {   //if running from command line and target folder doesn't exist
            throw new IOException("Failed to create build dir at " + buildDir.getAbsolutePath());
        }
        resultFile = new File(project.getBuild().getDirectory() + "/querytranslator/output.ser");
        outputDir = new File(project.getBuild().getDirectory() + "/querytranslator/");
        if (!outputDir.exists() && !outputDir.mkdir()) {
            throw new IOException("Failed to create the output directory in " + outputDir.getAbsolutePath());
        }
    }

    /**
     * Serialized the ResultVO to disk (the location is determined in the constructor).
     * 
     * @param result the ResultVO to serialize.
     * @throws IOException if failed to serialize.
     */
    public void serialize(final ResultVO result) throws IOException {
        if (resultFile.exists()) {
            if (!resultFile.delete()) {
                throw new IOException("Failed to delete old report at " + resultFile.getAbsolutePath());
            }
        }
        if (!resultFile.createNewFile()) {
            throw new IOException("Failed to create report file at " + resultFile.getAbsolutePath());
        }
        FileOutputStream fos = null;
        ObjectOutputStream out = null;
        fos = new FileOutputStream(resultFile);
        out = new ObjectOutputStream(fos);
        out.writeObject(result);
        out.close();
    }

    /**
     * Deserialized the ResultVO from disk (the location is determined in the constructor).
     * 
     * @return the deserialized ResultVO or null if the file doesn't exist.
     * @throws IOException if failed to deserialize.
     * @throws ClassNotFoundException if the ResultVO is not found on the classpath (should never occur).
     */
    public ResultVO deserialize() throws IOException, ClassNotFoundException {
        FileInputStream fis = null;
        ObjectInputStream in = null;
        if (!resultFile.exists()) {
            return null;
        }
        fis = new FileInputStream(resultFile);
        in = new ObjectInputStream(fis);
        final Object o = in.readObject();
        in.close();
        return (ResultVO) o;
    }
    
    public void textOutput(String fileName, String stringOutput) throws IOException {
        File outputFile = new File(outputDir.getAbsolutePath() + "/" + fileName);
        FileWriter writer = new FileWriter(outputFile);
        writer.write(stringOutput);
        writer.flush();
        writer.close();
    }
}
