/*******************************************************************************
 * Copyright 2012 pw999
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.oqt.maven;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.sf.oqt.core.CoreFactory;
import net.sf.oqt.jpa.EntityFinder;
import net.sf.oqt.jpa.JPATransformer;
import net.sf.oqt.model.EntityVO;
import net.sf.oqt.model.PackageVO;
import net.sf.oqt.model.QueryVO;
import net.sf.oqt.model.ResultVO;

import org.apache.derby.jdbc.EmbeddedDriver;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;

import com.google.common.base.Strings;

/**
 * This mojo will only validate the JPQL queries.
 * <p>
 * A database connection is not required to run this plugin, a Derby database connection is automatically initalized
 * (since Derby is supported by OpenJPA 1.0.0 to OpenJPA 2.2).
 * <p>
 * This plugin will make the build fail if not all JPQL queries are valid.
 * 
 * @goal validate
 * @phase validate
 * @author phillip
 * 
 */
public class QueryTranslatorValidateMojo extends AbstractMojo {

    /**
     * The list of packages where the plugin must go look for entities.
     * <p>
     * The plugin will automatically look in all the subpackages.
     * <p>
     * If this plugin is ran outside the model-module, then it's advised to give the model as a dependency of this
     * plugin.
     * 
     * @parameter
     * @required
     */
    private final List<String> packageNames = new ArrayList<String>();

    /**
     * Skips the validation.
     * 
     * @parameter default-value="false"
     * @required
     */
    private boolean skipValidation = false;
    
    public final List<String> getPackageNames() {
        return packageNames;
    }

    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        if (skipValidation) {
            getLog().info("Skipping validation because skipValidation was set to " + skipValidation);
            return;
        }
        getLog().info("Starting Validate Mojo");
        try {
            CoreFactory.getProperties().getPackageNames().clear(); // if you run this mojo twice ...
            CoreFactory.getProperties().getPackageNames().addAll(getPackageNames());
            CoreFactory.getProperties().setDictionary("derby");
            CoreFactory.getProperties().setDriverName(EmbeddedDriver.class.getName());
            CoreFactory.getProperties().setUrl("jdbc:derby:db/test/derby;create=true;databaseName=OQT_Derby");

            final ResultVO result = new ResultVO();
            result.getPackages().addAll(EntityFinder.findAllEntities(CoreFactory.getProperties().getPackageNames()));
            if (result.getPackages().isEmpty()) {
                getLog().warn("Nothing found, are the packages correctly configured ? ");
                return;
            }
            CoreFactory.getProperties().setFqnsFromResult(result);
            JPATransformer.validateOnly(result.getPackages());
            boolean failure = false;
            for (final PackageVO packageVO : result.getPackages()) {
                for (final EntityVO entityVO : packageVO.getEntities()) {
                    for (final QueryVO queryVO : entityVO.getQueries()) {
                        if (!Strings.isNullOrEmpty(queryVO.getError())) {
                            failure = true;
                            getLog().error("** Query Failure **");
                            getLog().error("Name: \t" + queryVO.getName());
                            getLog().error("JPQL: \t" + queryVO.getJPQLQuery());
                            getLog().error("Error:\t" + queryVO.getError());
                        }
                    }
                }
            }

            if (failure) {
                throw new MojoFailureException("Some JPQL queries could not be validated");
            }
            getLog().info("Finished Validate Mojo");

        } catch (ClassNotFoundException e) {
            throw new MojoExecutionException("ClassNotFoundException while executing validate mojo", e);
        } catch (InstantiationException e) {
            throw new MojoExecutionException("InstantiationException while executing validate mojo", e);
        } catch (IllegalAccessException e) {
            throw new MojoExecutionException("IllegalAccessException while executing validate mojo", e);
        } catch (SQLException e) {
            throw new MojoExecutionException("SQLException while executing validate mojo", e);
        } finally {
            try {
                CoreFactory.shutdown();
            } catch (final SQLException e) {
                getLog().error("Failed to shutdown database connection", e);
                throw new MojoExecutionException(e.getMessage());
            }
        }
    }

    public final boolean isSkipValidation() {
        return skipValidation;
    }

    public final void setSkipValidation(boolean skipValidation) {
        this.skipValidation = skipValidation;
    }

}
